Package: gdxrrw
Title: An Interface Between 'GAMS' and R
Version: 1.0.5
Depends: R (>= 2.14)
Date: 2019-09-27
SystemRequirements: GAMS (>= 24.2.1)
Authors@R: c(person("Steve","Dirkse", role=c("aut","cre"),
	             email="R@gams.com"),
             person("Michael","Ferris", role=c("aut")),
	     person("Rishabh","Jain", role=c("aut")) )
Description: A data interface between 'GAMS' and R.  The 'GAMS'
  (General Algebraic Modeling System) software includes a data
  specification called 'GDX' that is the preferred way to store and
  exchange 'GAMS' data.  This package includes several functions to
  transfer data between 'GDX' and R, and some related utility
  functions.
URL: http://www.gams.com
License: EPL2 with Secondary License GPL-2.0 or greater
Imports: reshape2
LazyData: no
Author: Steve Dirkse [aut, cre],
  Michael Ferris [aut],
  Rishabh Jain [aut]
Maintainer: Steve Dirkse <R@gams.com>
Built: R 3.6.1; x86_64-apple-darwin15.6.0; 2019-09-30 19:32:05 UTC; unix
